/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      COMBImodul-C14

  Description:  The module contents function for a tiny version of rtos.

  -------------------------------------------------------------------------

                $RCSfile: RtosX16x.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.2 $  $Date: 2004/10/28 13:00:59 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/02/12 -as:   creating 

  2004/09/24 -as:   Definition of tasks for using with PLC-LZS (XC161)

****************************************************************************/

#include <intrins.h>
#include <XC161.h>

#include "generic/global.h"
#include "../include/rtos.h"

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

// For timer controlled task the rtos uses the CAPTURE/COMPARE-Unit.
//
// -------------------------------------------------------------------------
// Task "PLL/OWD"
// This task is used to signaling errors of PLL. An error can only be dedected
// in clock direct or prescaler mode.
#define REG_IC_TASK_PLL    PLLIC
#define ENABLE_TASK_PLL    PLLIC_IE
#define PRIORITY_TASK_PLL  (ILVL_TASK_PLL << 2 | GLVL_TASK_PLL)

tfpRtosTaskHandler fpTaskHandler_PLL = (tfpRtosTaskHandler)NULL;

// -------------------------------------------------------------------------
// Task "Sys-Kernel Watchdog"
// This task is used to dedect an overrun of watchdog timer periode during
// software testing with monitor. If software runs in monitor the hardware
// watchdog is disabled. So this software watchdog can check up an overrun.
#define REG_IC_TASK_TMR_WDT    CC1_CC2IC
#define SIGNAL_TASK_TMR_WDT    CC1_CC2IC_IR
#define ENABLE_TASK_TMR_WDT    CC1_CC2IC_IE
#define REG_TM_TASK_TMR_WDT    CC1_M0
#define REG_Tx_TASK_TMR_WDT    CC1_CC2
#define MASK_TM_TASK_TMR_WDT   0x0F00 // select channel CC2
#define MODE_TM_TASK_TMR_WDT   0x0C00 // using T1, COMPARE-Mode 0
#define PRIORITY_TASK_TMR_WDT  (ILVL_TASK_TMR_WDT << 2 | GLVL_TASK_TMR_WDT | XGLVL_TASK_TMR_WDT << 8)

tfpRtosTaskHandler fpTaskHandler_TMR_WDT = (tfpRtosTaskHandler)NULL;

// -------------------------------------------------------------------------
// Task "ADC Overrun Error"
// This task is called if an overrun of ADDAT register bis occured. 
#define REG_IC_TASK_ADCE   ADC_EIC
#define ENABLE_TASK_ADCE   ADC_EIC_IE
#define PRIORITY_TASK_ADCE (ILVL_TASK_ADCE << 2 | GLVL_TASK_ADCE)

tfpRtosTaskHandler fpTaskHandler_ADCE = (tfpRtosTaskHandler)NULL;

// -------------------------------------------------------------------------
// Task "Kommunikationsprozess CANopen"
// This task is called every cycle time to process the CANopen communiction 
// process.
#define REG_IC_TASK_COP    CC1_CC3IC
#define SIGNAL_TASK_COP    CC1_CC3IC_IR
#define ENABLE_TASK_COP    CC1_CC3IC_IE
#define REG_TM_TASK_COP    CC1_M0
#define REG_Tx_TASK_COP    CC1_CC3
#define MASK_TM_TASK_COP   0xF000 // select channel CC3
#define MODE_TM_TASK_COP   0xC000 // use T1, COMPARE-Mode 0
#define PRIORITY_TASK_COP  (ILVL_TASK_COP << 2 | GLVL_TASK_COP)

static WORD wCopCriticalSections_l;

tfpRtosTaskHandler fpTaskHandler_COP = (tfpRtosTaskHandler)NULL;

// -------------------------------------------------------------------------
// globale defines

#define INT_ENABLE  0x40


//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Modul1>                                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Fr die Realisierung des RTOS werden die Eigenschaften des MC bezglich 
// seiner Interruptstruktur genutzt. Es knnen daher nur ereignisgesteuerte
// Task definiert werden. Eine Task ist stets an einen Interrupt gebunden.
// Als Ereignisse knnen nur Signale und Timeouts genutzt werden. Ein Signal
// entspricht dem Interrupt Request Flag, Timeouts knnen verwendet werden,
// wenn der benutzte Interrupt durch einen Timer realisiert wurde, d.h. die 
// Task die ISR eines Timers ist. Jede Task mu sich kooperativ verhalten. 
// D.h. Task mit einer geringeren Prioritt werden durch die Ausfhrung 
// einer Task mit einer hheren Prioritt gesperrt. Es wird kein Zeitscheiben-
// Verfahren untersttzt. Die Initialisierung einer Task, das Festlegen der 
// Prioritt mu durch Anpassen der Konstanten erfolgen. 
// Eine Task kann stets durch ein Signal aktiviert werden. Ein Signal wird 
// entweder unmittelbar durch die Hardware oder mit Hilfe einer Funktion
// gesetzt.
// Die Funktionen sind reentrant auszufhren, d.h. innerhalb der Funktionen
// drfen von verschiedenen Task keine statische Variablen benutzt werden. 
//
/***************************************************************************/

//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

// Definition des Basistimers fr das Erzeugen von timergesteuerten Task.
// Der Basistimer wird in Verbindung mit der Capture/Compare-Einheit
// verwendet. Bei der Definition einer timergesteuerten Task ist beim 
// Initialisieren des Compare-Kanals auf diesen Timer bezugzunehmen.

#define Tx                  CC1_T1    // timer register
#define TxCON               CC1_T01CON// timer control register
#define TxIC                CC1_T1IC  // interrupt control register
#define TxCON_MASK          0xFF00    // select only bits for timer 1
#define TxCFG               0x4700    // timer mode, timer run, Prescaler = 1024
                                      // (1 Tick = 25.6s bei 40MHz CPU-Takt)                                     

//---------------------------------------------------------------------------
#define TIME_PER_TICK       1638      // Um kleineren Rundungsfehler zu erhalten 
                                      // wird der Wert skalliert: 25.6 * 64
#define TICKS_PER_MILLISEC  39        // timer ticks fr 0,9984ms 
                                      // (1 Tick = 25.6s bei 40MHz CPU-Takt)

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

// -------------------------------------------------------------------------
//
// Function:     RtosInit
//
// Description:  Die Funktion initialisiert den Timer fr zeitgesteuerte 
//               Ablufe. Der Timer luft unabhngig vom Rest der Anwendung
//               und stellt eine Zeitbasis zur Verfgung, die benutzt werden
//               kann fr die COMPARE-Funktion der CAPTURE-Unit. Auf diese 
//               Weise knnen bis zu 16 zeitgesteuerte Prozess generiert werden.
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        getestet (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

void PUBLIC RtosInit (void)
{
    Tx     = 0;
    TxIC   = 0;
    TxCON |= (TxCON_MASK & TxCFG);     // Timer RUN, Timermode
}

// -------------------------------------------------------------------------
//
// Function:     RtosShutDown
//
// Description:  Die Funktion prft, ob alle installierbaren Handler der App
//               disabled wurden. Anderfalls kann es passieren, da ein Interrupt
//               ausgelst und der Handler gestartet wird, jedoch keine Applikation  
//               vorhanden ist.
//
// Parameters:   ShutDownMode:
//               RTOS_SHUTDOWN_SYS_LEVEL:  
//               Es wird berprft, ob smtliche Handler deinstalliert worden.
//
//               RTOS_SHUTDOWN_USER_LEVEL: 
//               Es wird berprft, ob die User-Handler deinstalliert worden.
//
// 
// Returns:      Errorcode
//
//
// State:        getestet (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

BYTE PUBLIC RtosShutDown (BYTE bShutDownMode_p)
{
BYTE bErr = kRtosSuccessful;

    if (bShutDownMode_p == RTOS_SHUTDOWN_USER_LEVEL)
    {
        // Sollte eines der Handler noch installiert sein,
        // dann den Handler lschen, so da eine Interruptannahme
        // nicht zum Aufruf des Handlers fhrt.
        if (
            //----------------------------------------------------------------
            // TASK_TMR_COM:
            fpTaskHandler_COP ||
            //----------------------------------------------------------------
            // TASK_ADC_ERROR:
            fpTaskHandler_ADCE ||
            //----------------------------------------------------------------
            // TASK_TMR_WDT:
            fpTaskHandler_TMR_WDT ||
            //----------------------------------------------------------------
            // TASK_PLL:
            fpTaskHandler_PLL 
            
            )
        {        
           //----------------------------------------------------------------
           bErr = kRtosFatalError;

           // Alle Handler lschen:
           //----------------------------------------------------------------
           // TASK_TMR_COM:
           RtosDeleteTask(TASK_TMR_COM);
           //----------------------------------------------------------------
           // TASK_ADC_ERROR:
           RtosDeleteTask(TASK_ADC_ERROR);
           //----------------------------------------------------------------
           // TASK_TMR_WDT:
           RtosDeleteTask(TASK_TMR_WDT);
           //----------------------------------------------------------------
           // TASK_PLL_ERROR:
           RtosDeleteTask(TASK_PLL_ERROR);
        }
    }
    else
    {
        // RTOS_SHUTDOWN_SYS_LEVEL
        if (
            //----------------------------------------------------------------
            // TASK_TMR_COM:
            fpTaskHandler_COP ||
            //----------------------------------------------------------------
            // TASK_ADC_ERROR:
            fpTaskHandler_ADCE ||
            //----------------------------------------------------------------
            // TASK_TMR_WDT:
            fpTaskHandler_TMR_WDT ||
            //----------------------------------------------------------------
            // TASK_PLL:
            fpTaskHandler_PLL 
            
            )
            //----------------------------------------------------------------
            bErr = kRtosFatalError;
    }

    return (bErr);
}


// -------------------------------------------------------------------------
//
// Function:     RtosCreateTask
//
// Description:  Die Funktion initialisiert eine Task. D.h. es wird
//               die Interruptprioritt gesetzt und der Interrupt 
//               freigegeben. Je nach Task kann ein Task-Handler der
//               Applikation parametriert werden.
//
//               Bei einer timergesteuerten Task wird das vorgesehene
//               Compareregister init. 
//
// Parameters:   Nummer der Task
//
// 
// Returns:      Fehlercode
//
//
// State:        getestet (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

BYTE PUBLIC RtosCreateTask (BYTE bTaskNo_p, tfpRtosTaskHandler fpTaskHandler_p)
{
BYTE bErr = kRtosSuccessful;

    RtosDeleteTask(bTaskNo_p);

    switch (bTaskNo_p)
    {
        //----------------------------------------------------------------
        // Kommunikations-Task
         case TASK_TMR_COM:
            // -----------------------------------------------------
            // Startwert fr ersten Interrupt setzen
            REG_Tx_TASK_COP  = Tx + TICKS_PER_MILLISEC;
            // Timer starten: Compare-Register init
            REG_TM_TASK_COP &= ~MASK_TM_TASK_COP;
            REG_TM_TASK_COP |= MODE_TM_TASK_COP;
            // Task-Handler der Applikation einbinden
            fpTaskHandler_COP = fpTaskHandler_p;
            // Interrupt freigeben
            REG_IC_TASK_COP = INT_ENABLE + PRIORITY_TASK_COP;
        break;
        //----------------------------------------------------------------
        // ADC Overrun Error Task
        case TASK_ADC_ERROR:
            // Task-Handler der Applikation einbinden
            fpTaskHandler_ADCE = fpTaskHandler_p;
            // Interrupt freigeben
            REG_IC_TASK_ADCE   = INT_ENABLE + PRIORITY_TASK_ADCE;   
        break;

        //----------------------------------------------------------------
        // Watchdog Timer
        case TASK_TMR_WDT:
            // Handler installieren
            fpTaskHandler_TMR_WDT = fpTaskHandler_p;
            // Timer starten: Compare-Register init, Interrupt enablen
            REG_Tx_TASK_TMR_WDT  = Tx - 1;
            REG_TM_TASK_TMR_WDT &= ~MASK_TM_TASK_TMR_WDT;
            REG_TM_TASK_TMR_WDT |= MODE_TM_TASK_TMR_WDT;
            REG_IC_TASK_TMR_WDT  = INT_ENABLE + PRIORITY_TASK_TMR_WDT;
        break;

        //----------------------------------------------------------------
        // Interrupt PLL/OWD
        case TASK_PLL_ERROR:
            // Callback-Handler der Applikation einbinden
            fpTaskHandler_PLL = fpTaskHandler_p;
            // Interrupt freigeben
            REG_IC_TASK_PLL   = INT_ENABLE + PRIORITY_TASK_PLL;
        break;

        //----------------------------------------------------------------
        default:  
            bErr = kRtosUnvalidTaskNo;
        break;
    }

    return (bErr);
}


// -------------------------------------------------------------------------
//
// Function:     RtosDeleteTask
//
// Description:  Die Funktion sperrt den Interrupt einer Task. Dadurch wird 
//               die Task nicht erneut gestartet.
//
//
// Parameters:   Nummer der Task
//
// 
// Returns:      Fehlercode
//
//
// State:        getestet (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

BYTE PUBLIC RtosDeleteTask (BYTE bTaskNo_p)
{
BYTE bErr = kRtosSuccessful;

    switch (bTaskNo_p)
    {
        //----------------------------------------------------------------
        // Kommunikations-Task
        case TASK_TMR_COM:
            // Timer sperren
            REG_TM_TASK_COP &= ~MASK_TM_TASK_COP;
            // Interrupt sperren
            REG_IC_TASK_COP = 0;
            // Anzahl Critical Sections zurcksetzen
            wCopCriticalSections_l = 0;
            // Task-Handler der Applikation lschen
            fpTaskHandler_COP = (tfpRtosTaskHandler)NULL;
        break;
        //----------------------------------------------------------------
        // ADC Overrun Error Task
        case TASK_ADC_ERROR:
            // Interrupt sperren
            REG_IC_TASK_ADCE   = 0;   
            // Task-Handler der Applikation lschen
            fpTaskHandler_ADCE = (tfpRtosTaskHandler)NULL;
        break;

        //----------------------------------------------------------------
        // Watchdog Timer
        case TASK_TMR_WDT:
            REG_TM_TASK_TMR_WDT &= ~MASK_TM_TASK_TMR_WDT;
            REG_IC_TASK_TMR_WDT  = 0;
            // Handler deinstallieren
            fpTaskHandler_TMR_WDT = (tfpRtosTaskHandler)NULL;

        break;

        //----------------------------------------------------------------
        // Interrupt PLL/OWD
        case TASK_PLL_ERROR:
            // Interrupt deaktivieren
            REG_IC_TASK_PLL   = 0;
            // Task-Handler der Applikation lschen
            fpTaskHandler_PLL = (tfpRtosTaskHandler)NULL;
        break;

        //----------------------------------------------------------------
        default:  
            bErr = kRtosUnvalidTaskNo;
        break;
    }

    return (bErr);
}

// -------------------------------------------------------------------------
//
// Function:     RtosDelayTask
//
// Description:  Die Funktion kann benutzt werden, die Sperrzeit fr eine 
//               mit Hilfe eines Timers gesteuerte Task zu setzen.  
//
//
// Parameters:   Task-Nummer
//               Anzahl Microsekunden bis zur nchsten Ausfhrung
//
// 
// Returns:      Fehlercode
//
//
// State:        getestet (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

BYTE PUBLIC RtosDelayTask (BYTE bTaskNo_p, DWORD dwTimeout_p) 
{
BYTE bErr = kRtosSuccessful;
WORD wTimerTicks;

    // Um einen kleineren Rundungsfehler zu erhalten, wird
    // der Parameter skalliert mit 64.
    wTimerTicks = (WORD)((dwTimeout_p << 6) / TIME_PER_TICK) + 1;

    switch (bTaskNo_p)
    {
        //----------------------------------------------------------------
        // Kommunikations-Task
        case TASK_TMR_COM:
            // neuen Compare-Wert fr Timer setzen
            _atomic_(0);
            REG_Tx_TASK_COP   = Tx + wTimerTicks; 
            _endatomic_();
        break;

        //----------------------------------------------------------------
        // Watchdog Timer
        case TASK_TMR_WDT:
            _atomic_(4);
            // Ein inzwischen gesetztes Interrupt-Flag lschen.
            SIGNAL_TASK_TMR_WDT  = 0;
            _atomic_(0);
            REG_Tx_TASK_TMR_WDT  = Tx + wTimerTicks;
            _endatomic_();
        break;

        //----------------------------------------------------------------
        // ADC Overrun Error Task
        case TASK_ADC_ERROR:
        // Interrupt PLL/OWD
        case TASK_PLL_ERROR:
            // Hier gibt es nichts zu tun, diese Tasks werden 
            // nicht durch Timer verzgert.
        break;

        //----------------------------------------------------------------
        default:  
            bErr = kRtosUnvalidTaskNo;
        break;
    }

    return (bErr);
}

// -------------------------------------------------------------------------
//
// Function:     RtosSendSignal
//
// Description:  Die Funktion setzt das Interrupt Request Flag einer Task
//               und lst damit Interrupt aus. Die ISR (Task) wird ausgefhrt,
//               wenn keine hher priorisierte Task aktiv ist. 
//
//
// Parameters:   Task-Nummer
//
// 
// Returns:      Fehlercode
//
//
// State:        getestet (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

BYTE PUBLIC RtosSendSignal (BYTE bTaskNo_p)
{

BYTE bErr = kRtosSuccessful;

    switch (bTaskNo_p)
    {
        //----------------------------------------------------------------
        // Kommunikations-Task
        case TASK_TMR_COM:
            SIGNAL_TASK_COP = 1;
        break;

        //----------------------------------------------------------------
        // ADC Overrun Error Task
        case TASK_ADC_ERROR:
        // Watchdog Timer
        case TASK_TMR_WDT:
        // Interrupt PLL/OWD
        case TASK_PLL_ERROR:
            // Ein Signal darf nur durch die Hardware gesendet werden. 
        break;

        //----------------------------------------------------------------
        default:  
            bErr = kRtosUnvalidTaskNo;
        break;
    }

    return (bErr);
}

// -------------------------------------------------------------------------
//
// Function:     RtosClearSignal
//
// Description:  Die Funktion lscht ein Interrupt Request Flag einer Task.
//               Die Anwendung der Funktion ist dann sinnvoll, wenn eine 
//               laufende Task durch eine hherpriorisierte Task erneut signalisiert 
//               wird. Das Interrupt Requestflag wird automatisch beim Eintritt 
//               in die ISR gelscht. Wird jedoch whrend der Ausfhrung der ISR die
//               Task erneut signalisiert, so wird die Task unmittelbar nach dem 
//               Beenden erneut gerufen. Das kann unter Umstnden unerwnscht sein. 
//
//
// Parameters:   Task-Nummer
//
// 
// Returns:      Fehlercode
//
//
// State:        getestet (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

BYTE PUBLIC RtosClearSignal (BYTE bTaskNo_p)
{

BYTE bErr = kRtosSuccessful;

    switch (bTaskNo_p)
    {
        //----------------------------------------------------------------
        // Kommunikations-Task
        case TASK_TMR_COM:
            SIGNAL_TASK_COP = 0;
        break;

        //----------------------------------------------------------------
        // Watchdog Timer
        case TASK_TMR_WDT:
            SIGNAL_TASK_TMR_WDT = 0;
        break;

        //----------------------------------------------------------------
        // ADC Overrun Error Task
        case TASK_ADC_ERROR:
        // Interrupt PLL/OWD
        case TASK_PLL_ERROR:
            // Ein Signal darf nur durch die Hardware gelscht werden. 
        break;

        //----------------------------------------------------------------
        default:  
            bErr = kRtosUnvalidTaskNo;
        break;
    }

    return (bErr);
}

// -------------------------------------------------------------------------
//
// Function:     RtosDisableTask
//
// Description:  Die Funktion sperrt die Ausfhrung einer Task.
//
//
// Parameters:   Task-Nummer
//
// 
// Returns:      Fehlercode
//
//
// State:        getestet (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

BYTE PUBLIC RtosDisableTask (BYTE bTaskNo_p)
{
BYTE bErr = kRtosSuccessful;

    switch (bTaskNo_p)
    {
        //----------------------------------------------------------------
        // Kommunikations-Task
        case TASK_TMR_COM:
           ENABLE_TASK_COP = 0;
           // Jede Sperre mitzhlen...
           wCopCriticalSections_l++;
        break;

        //----------------------------------------------------------------
        // ADC Overrun Error Task
        case TASK_ADC_ERROR:
            ENABLE_TASK_ADCE = 0;
        break;

        //----------------------------------------------------------------
        // Watchdog Timer
        case TASK_TMR_WDT:
            ENABLE_TASK_TMR_WDT  = 0;
        break;

        //----------------------------------------------------------------
        // Interrupt PLL/OWD
        case TASK_PLL_ERROR:
            ENABLE_TASK_PLL   = 0;
        break;

        //----------------------------------------------------------------
        default:  
           bErr = kRtosUnvalidTaskNo;
        break;
    }

    return (bErr);
}

// -------------------------------------------------------------------------
//
// Function:     RtosEnableTask
//
// Description:  Die Funktion gibt die Ausfhrung einer Task wieder frei.
//
//
// Parameters:   Task-Nummer
//
// 
// Returns:      Fehlercode
//
//
// State:        getestet (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

BYTE PUBLIC RtosEnableTask (BYTE bTaskNo_p)
{
BYTE bErr = kRtosSuccessful;

    switch (bTaskNo_p)
    {
        //----------------------------------------------------------------
        // Kommunikations-Task
        case TASK_TMR_COM:
           // Jede Freigabe mitzhlen...
           wCopCriticalSections_l--;
           // Nur wenn kein kritischer Abschnitt,
           // dann freigeben.
           if (!wCopCriticalSections_l)
           {
              ENABLE_TASK_COP = 1;
           }
        break;

        //----------------------------------------------------------------
        // ADC Overrun Error Task
        case TASK_ADC_ERROR:
            ENABLE_TASK_ADCE = 1;
        break;

        //----------------------------------------------------------------
        // Watchdog Timer
        case TASK_TMR_WDT:
            ENABLE_TASK_TMR_WDT  = 1;
        break;

        //----------------------------------------------------------------
        // Interrupt PLL/OWD
        case TASK_PLL_ERROR:
            ENABLE_TASK_PLL   = 1;
        break;

        //----------------------------------------------------------------
        default:  
           bErr = kRtosUnvalidTaskNo;
        break;
    }

    return (bErr);
}


//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//
//
// ISRn fr den CAN-Controller sind innerhalb der Module implementiert. 
// ISRn fr HW-Fehlerereignisse sind in diesem
// Modul implementiert und knnen Ereignisse an einen applikationsspez.
// Handler weiterleiten. Dazu ist beim Initialisieren einer Task die 
// Adresse des Handlers als Parameter zu bergeben. 
//
//---------------------------------------------------------------------------
//
// Function:     ISR_TMR_COM
//
// Description:  Trap Handler for TME COM interrupts.
//
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        tested (24.09.2004 A.S.)
//
//---------------------------------------------------------------------------

static void ISR_TMR_COM (void) OS_TASK TASK_TMR_COM
{
    if (fpTaskHandler_COP)
       fpTaskHandler_COP();
}

// -------------------------------------------------------------------------
//
// Function:     ISR_ADC_ERROR
//
// Description:  Die Funktion wird ausgefhrt, wenn ein Fehler bei der 
//               AD-Wandlung aufgetreten ist. 
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        tested (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

static void ISR_ADC_ERROR (void) OS_TASK TASK_ADC_ERROR
{
    if (fpTaskHandler_ADCE)
       fpTaskHandler_ADCE();
}

//---------------------------------------------------------------------------
//
// Function:     ISR_PLL_ERROR
//
// Description:  Trap Handler for PLL/OWD interrupts.
//
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

static void ISR_PLL_ERROR (void) OS_TASK TASK_PLL_ERROR
{
    if (fpTaskHandler_PLL)
       fpTaskHandler_PLL();
}

//---------------------------------------------------------------------------
//
// Function:     ISR_WdtTimer
//
// Description:  Die Funktion wird aufgerufen, wenn die Perioden des 
//               Watchdogs angelaufen ist. 
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        tested (24.09.2004 A.S.)
//
// -------------------------------------------------------------------------

static void ISR_WdtTimer (void) OS_TASK TASK_TMR_WDT
{
  
    if (fpTaskHandler_TMR_WDT)
       fpTaskHandler_TMR_WDT();
}


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <SysKernel Watchdog>                                */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Diese Klasse definiert Funktionen zur Diagnose der Watchdog-Funktion. 
// Dadurch ist es mglich, Programmteile zu ermitteln, die durch eine 
// Blockierung der Prozessfunktion verhindern, das Watchdog-Timeout rechtzeitig
// zurckzusetzen. 
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define SYSKERNEL_WDT_TIMEOUT   1300000L    // Watchdog-Timeout 400ms

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

// -------------------------------------------------------------------------
//
// Function:     RtosInitWatchdog
//
//
// Description:  Die Funktion init. den Softwarewatchdog.
//
//
// Parameters:   Applikations-Handler
//
// 
// Returns:      keine
//
//
// State:        getestet (12.08.2003 A.S.)
//
// -------------------------------------------------------------------------

void PUBLIC RtosInitWatchdog (tfpRtosTaskHandler fpTaskHandler_p)
{
    // Task definieren
    RtosCreateTask(TASK_TMR_WDT, fpTaskHandler_p);

    // Verzgerung setzen
    RtosDelayTask(TASK_TMR_WDT, SYSKERNEL_WDT_TIMEOUT);
}

// -------------------------------------------------------------------------
//
// Function:     RtosShutDownWatchdog
//
//
// Description:  Die Funktion beendet den Softwarewatchdog.
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        getestet (12.08.2003 A.S.)
//
// -------------------------------------------------------------------------

void PUBLIC RtosShutDownWatchdog (void)
{
    // Task beenden
    RtosDeleteTask(TASK_TMR_WDT);

}


// -------------------------------------------------------------------------
//
// Function:     RtosResetWatchdog
//
//
// Description:  Die Funktion berwacht die Watchdog-Periodendauer und
//               setzt das Timeout zurck. In der Debugversion kann im 
//               Falle eines berschreitens des Timeouts die Verarbeitung
//               unterbrochen werden. 
//
//
// Parameters:   keine
//
// 
// State:        getestet (12.08.2003 A.S.)
//
// -------------------------------------------------------------------------

void PUBLIC RtosResetWatchdog (void)
{
WORD wTaskPriority;

    // Nur auf niedrigster Priorittsebene
    // darf der Watchdog zurckgesetzt werden.
    wTaskPriority = PSW >> 12;

    if (wTaskPriority == 0)
    {
       _srvwdt_();

       RtosDelayTask(TASK_TMR_WDT, SYSKERNEL_WDT_TIMEOUT);
    }
}

// -------------------------------------------------------------------------
//
// Function:     RtosClearWatchdogEvent
//
//
// Description:  Die Funktion setzt den Watchdog zurck und lscht 
//               das Interruptflag fr den Watchdog falls globaler
//               Interrupt gesperrt ist.
//
//
// Parameters:   keine
//
// 
// State:        getestet (12.08.2003 A.S.)
//
// -------------------------------------------------------------------------

void PUBLIC RtosClearWatchdogEvent (void)
{
    RtosResetWatchdog();

    RtosClearSignal(TASK_TMR_WDT);
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

// ...
